// Copyright (c) Microsoft Corporation. All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//

#include "appwindow.h"
#include <atlbase.h>
#include <streams.h>
#include <initguid.h>
#include <imgguids.h>
#include <iostream>
using namespace std;

#include "CPropertyBag.h"
#include "../VideoCapFilter/VideoCapFilter.h"
#include "..//commands.h"

#define CHK( x ) do{ if( FAILED( hr = ( x ))) { goto Cleanup; }} while( FALSE );
#define ERR( x ) do{ hr = x; goto Cleanup; } while( FALSE );

void RunVideoStream( void );
HRESULT InitCaptureGraphBuilder( IGraphBuilder **ppGraph, ICaptureGraphBuilder2 **ppBuild );
HRESULT GetFirstCameraDriver( WCHAR *pwzName );
HRESULT InitCaptureFilter ( IGraphBuilder *pGraph, CComPtr<IBaseFilter> &pCap );
HRESULT InitVideoCapFilter ( IGraphBuilder *pGraph, CComPtr<IBaseFilter> &DTMFilter );
HRESULT RunCaptureGraph( ICaptureGraphBuilder2 *pBuild );
void    StudyFilterPins( CComPtr<IBaseFilter> &pFilter );
void EmbedVideoInWindow(void);
void HandleEvent(void);


#define WINDOWWIDTH 240
#define WINDOWHEIGHT 240
#define CMD_START_RECORD_VIDEO 1
#define CMD_STOP_RECORD_VIDEO 2
#define CMD_STILL_IMAGE 3


#define WM_GRAPHNOTIFY  WM_APP + 1
#define CLASSNAME "EventNotify"

//IGraphBuilder   *pGraph = NULL;
IMediaControl   *pMediaControl = NULL;
IMediaEventEx   *pEvent = NULL;
HWND            g_hwnd;

RECT        rcWnd={0,0,WINDOWWIDTH,WINDOWHEIGHT};
RECT        rcImg={0,0,0,0};


HWND hwnd;

IGraphBuilder *pGraph;			// The filter graph.
ICaptureGraphBuilder2 *g_pBuild;	// Capture Graph Builder
CComQIPtr<IBaseFilter> pCap;		// Video capture filter.
CComQIPtr<IVideoCapMobileFilter> m_VideoCapFilter;
CComPtr<IBaseFilter> DTMFilter;	// Video capture filter.


CAppWindow::CAppWindow()
{
    m_hwnd   = NULL;
    m_button = NULL;

	current_cursor_x=20;
	current_cursor_y=20;
	target_cursor_x=20;
	target_cursor_y=20;
	clean_background=1;

	pImage = NULL;
	pImgFactory = NULL;
	mSclFac = 20;
}


CAppWindow::~CAppWindow()
{
    if( m_hwnd )
    {
        DestroyWindow( m_hwnd );
    }

    if( m_button )
    {
        DestroyWindow( m_button );
    }

    if( m_wzStatusString )
    {
        LocalFree( m_wzStatusString );
    }

    //if( m_pGraphManager )
    //{
    //    delete m_pGraphManager;
    //}
}


HRESULT
CAppWindow::CreateControls( HINSTANCE hInstance )
{
    HRESULT     hr = S_OK;
    WNDCLASS    wndClass;
    ATOM        classAtom;
    LONG     	lReturn;
	RECT		rcWorkArea;

    // First, let's register the window class
    ZeroMemory( &wndClass, sizeof( wndClass ));
    wndClass.lpfnWndProc = CAppWindow::AppWindowProc; 
    wndClass.hInstance = hInstance;
    wndClass.lpszClassName = L"Camera Recording Video Class";

    classAtom = RegisterClass( &wndClass );
    if( classAtom == NULL )
    {
        ERR( HRESULT_FROM_WIN32( GetLastError() ));
    }


    // Now, create the main window
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rcWorkArea, 0);

    m_hwnd = CreateWindow( 
			L"Camera Recording Video Class", 
			L"Main Window", 
			WS_VISIBLE, 
			rcWorkArea.left,
            rcWorkArea.top,
            rcWorkArea.right - rcWorkArea.left,
            rcWorkArea.bottom - rcWorkArea.top,
			NULL, 
			NULL, 
			hInstance, 
			(PVOID) this );

    if( m_hwnd == NULL )
    {
        ERR( HRESULT_FROM_WIN32( GetLastError() ));
    }

	g_hwnd = m_hwnd;
    
    // Workaround for bug in SetWindowLong, call twice the API
    lReturn = SetWindowLong( m_hwnd, GWL_USERDATA, (LONG) this );
    lReturn = SetWindowLong( m_hwnd, GWL_USERDATA, (LONG) this );
    if( lReturn == NULL )
    {
        ERR( HRESULT_FROM_WIN32( GetLastError() ));
    }

    // Create a "Start Capture" button and add it to the main window
   // m_button = CreateWindow( 
			//L"Button", 
			//L"Process Now", 
			//WS_CHILD | WS_VISIBLE | BS_CENTER | BS_PUSHBUTTON | BS_VCENTER, 
			//45, 
			//160, 
			//145, 
			//20, 
			//m_hwnd, 
			//(HMENU) 1, //  EA - dummy constant for now 
			//hInstance, 
			//NULL );

   // if( m_button == NULL )
   // {
   //     ERR( HRESULT_FROM_WIN32( GetLastError() ));
   // }

	//caglayan adding
	// Normally you would only call CoInitialize/CoUninitialize
    // once per thread.  This sample calls CoInitialize in this
    // draw function simply to illustrate that you must call 
    // CoInitialize before calling CoCreateInstance.
    CoInitializeEx(NULL, COINIT_MULTITHREADED);

	if (SUCCEEDED(CoCreateInstance (CLSID_ImagingFactory, NULL, CLSCTX_INPROC_SERVER,
                                    IID_IImagingFactory,(void **)&pImgFactory)))
	{
		if (SUCCEEDED(pImgFactory->CreateImageFromFile(
						TEXT("\\My Documents\\My Pictures\\boun.jpg"), &pImage)))
		{
			

		}

	}

	//HWND g_hWndMenuBar;

	//SHMENUBARINFO mbi;
	//memset(&mbi, 0, sizeof(SHMENUBARINFO));
	//mbi.cbSize     = sizeof(SHMENUBARINFO);
	//mbi.hwndParent = m_hwnd;
	////mbi.nToolBarId = IDR_MENU;
	////mbi.hInstRes   = g_hInst;
	//g_hWndMenuBar = mbi.hwndMB;

	//if(g_hWndMenuBar){
	//
	//	RECT rc;
 //       RECT rcMenuBar;
	//	

 //       GetWindowRect(m_hwnd, &rc);
 //       GetWindowRect(g_hWndMenuBar, &rcMenuBar);
 //       rc.bottom -= (rcMenuBar.bottom - rcMenuBar.top);
 //       
 //       MoveWindow(m_hwnd, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top, FALSE);
	//	rc_wnd = rc;
	//	rc_wnd.right -= rc_wnd.left;
	//	rc_wnd.left = 0;
	//	rc_wnd.bottom -= rc_wnd.top;
	//	rc_wnd.top = 0;
	//}
	
	

 //   // Create a "Stop Capture" button and add it to the main window
 //   m_stopButtonHwnd = CreateWindow( 
	//		L"Button", 
	//		L"Stop Video Recording", 
	//		WS_CHILD | WS_VISIBLE | BS_CENTER | BS_PUSHBUTTON | BS_VCENTER, 
	//		30, 
	//		40, 
	//		150, 
	//		20, 
	//		m_hwnd, 
	//		(HMENU) CMD_STOP_RECORD_VIDEO, 
	//		hInstance, 
	//		NULL );

 //   if( m_startButtonHwnd == NULL )
 //   {
 //       ERR( HRESULT_FROM_WIN32( GetLastError() ));
 //   }
 //   
 //   // The stop capture button is disabled by default
 //   EnableWindow( m_stopButtonHwnd, FALSE );

	//// Create the take still image button
	//m_stillButtonHwnd = CreateWindow( 
	//		L"Button", 
	//		L"Take a still image", 
	//		WS_CHILD | WS_VISIBLE | BS_CENTER | BS_PUSHBUTTON | BS_VCENTER, 
	//		30, 
	//		70, 
	//		150, 
	//		20, 
	//		m_hwnd, 
	//		(HMENU) CMD_STILL_IMAGE, 
	//		hInstance, 
	//		NULL );

 //   if( m_startButtonHwnd == NULL )
 //   {
 //       ERR( HRESULT_FROM_WIN32( GetLastError() ));
 //   }

    // Create the graph manager. This will control the dshow capture pipeline
    //m_pGraphManager = new CGraphManager();
    //if( m_pGraphManager == NULL )
    //{
    //    ERR( E_OUTOFMEMORY );
    //}

    //CHK( m_pGraphManager->RegisterNotificationWindow( m_hwnd ));

    //CHK( m_pGraphManager->Init());
    //CHK( m_pGraphManager->BuildCaptureGraph());

Cleanup:
    return hr; 
}


LRESULT CALLBACK
CAppWindow::AppWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    CAppWindow *pThis;

    //Let's retrieve the pointer to this first
    pThis = (CAppWindow*) GetWindowLong( hwnd, GWL_USERDATA );
    if( pThis )
    {
        return pThis->HandleWindowProc( hwnd, uMsg, wParam, lParam );
    }

    return DefWindowProc( hwnd, uMsg, wParam, lParam );
}


LRESULT 
CAppWindow::HandleWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
    HRESULT hr = S_OK;

    switch( uMsg )
    {
        case WM_PAINT:
            RepaintWindow();
            break;

        case WM_CLOSE:
            PostMessage( NULL, WM_QUIT, 0, 0 );
            return 0;
            break;

        case WM_USER:
            UpdateMessage( L"TESTING" ); // DUMMY MESSAGE, FOR NOW
            break;

		case WM_ACTIVATE:
			if( LOWORD( wParam ) == WA_INACTIVE )
			{
				PostMessage( NULL, WM_QUIT, 0, 0 );
			}

        case WM_COMMAND:
			HandleCommand( wParam );
            break;

        case WM_GRAPHNOTIFY:
            HandleEvent();
            break;

        default:
            return DefWindowProc( hwnd, uMsg, wParam, lParam );
    }

    return 0;
}


HRESULT
CAppWindow::HandleCommand( WPARAM wParam )
{
	HRESULT hr = S_OK;

//	if( m_pGraphManager == NULL )
//	{
//		ERR( E_FAIL );
//	}
//
//	switch( wParam )
//	{
//		case CMD_START_RECORD_VIDEO:
//			EnableWindow( m_startButtonHwnd, FALSE );
//			EnableWindow( m_stopButtonHwnd, TRUE );
//			EnableWindow( m_stillButtonHwnd, FALSE );
//			m_pGraphManager->StartRecordVideo();
//			break;
//
//		case CMD_STOP_RECORD_VIDEO:
//			EnableWindow( m_stopButtonHwnd, FALSE );
//			m_pGraphManager->StopRecordVideo();
//			break;
//
//		case CMD_STILL_IMAGE:
//			EnableWindow( m_startButtonHwnd, FALSE );
//			EnableWindow( m_stopButtonHwnd, FALSE );
//			EnableWindow( m_stillButtonHwnd, FALSE );
//			m_pGraphManager->CaptureStillImage();
//			break;
//
//		default:
//			break;
//    }
//
//Cleanup:
	return hr;
}


HRESULT
CAppWindow::RepaintWindow()
{
	
    HDC         hdc;
    HBRUSH      hBrush, hOldBrush;
    HPEN        hPen, hOldPen;
	//RECT        rcWnd={0,0,WINDOWWIDTH,WINDOWHEIGHT};
	//RECT		rcImg={target_cursor_x*mSclFac,
	//					target_cursor_y*mSclFac,
	//					target_cursor_x*mSclFac + WINDOWWIDTH*mSclFac,
	//					target_cursor_y*mSclFac + WINDOWHEIGHT*mSclFac};
	rcImg.left = target_cursor_x*mSclFac;
	rcImg.top = target_cursor_y*mSclFac;
	rcImg.right = target_cursor_x*mSclFac + WINDOWWIDTH*mSclFac;
	rcImg.bottom = target_cursor_y*mSclFac + WINDOWHEIGHT*mSclFac;
    PAINTSTRUCT ps;


    // The window is repainted in white
    hdc = BeginPaint( m_hwnd, &ps );

	hBrush = (HBRUSH) GetStockObject( WHITE_BRUSH );
    hPen = (HPEN) GetStockObject( WHITE_PEN );
    hOldBrush = (HBRUSH) SelectObject( hdc, hBrush );
    hOldPen   = (HPEN)   SelectObject( hdc, hPen );

	if (clean_background)
	{
		clean_background=0;
		Rectangle( hdc, 0, 0, WINDOWWIDTH, WINDOWHEIGHT );
	}
    
	hPen = (HPEN) GetStockObject( BLACK_PEN );
    SelectObject( hdc, hPen );
	hBrush = (HBRUSH) GetStockObject( BLACK_BRUSH );
    SelectObject( hdc, hBrush );


    // And let's add the status string on the window
    if( m_wzStatusString )
    {
        //DrawText( hdc, m_wzStatusString, -1, &rcText , DT_LEFT );
    }

	//POINT Point;
	//MoveToEx(hdc,current_cursor_x,current_cursor_y, &Point);
	//LineTo(hdc,target_cursor_x,target_cursor_y);

	current_cursor_x=target_cursor_x;
	current_cursor_y=target_cursor_y;
	
	SelectObject( hdc, hOldPen );
	SelectObject( hdc, hOldBrush );

	//draw the image
	//GetWindowRect(m_hwnd,&rcImage);
	pImage->Draw(hdc,&rcWnd,&rcImg);

    EndPaint( m_hwnd, &ps );

    return S_OK;
}



HRESULT CAppWindow::Run()
{
    HRESULT hr = S_OK;
    MSG msg;

    // Start the capture graph
    //CHK( m_pGraphManager->RunCaptureGraph());
	RunVideoStream();

	int deltax, deltay , id, command;
	int old_id=0;

	int simulate=0;

	while( true)
    {
		
		if (PeekMessage( &msg, NULL, 0, 0,PM_REMOVE ))
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		
		if (m_VideoCapFilter)
			{
				//get computed parameters
				m_VideoCapFilter->get_RecognitionResults( &deltax, &deltay, &id, &command );
				if (old_id!=id) // if these are newly computed values perform this
				{
					//OGEM process parameters deltax, deltay, and command here
					//Sleep(30);
					//simulate+=3;
					//target_cursor_x+=10;
					//target_cursor_y+=0;
					//target_cursor_x += rand()%6-3;
					//target_cursor_y += rand()%6-3;
					target_cursor_x = deltax*2;
					target_cursor_y = deltay*2;
					//cerr<<"tx:"<<target_cursor_x<<"ty:"<<target_cursor_y<<endl;
					InvalidateRect( m_hwnd, NULL, TRUE );

					if (command==SET_PEN_THICKNESS_TO_ONE)
					{
						//change thickness to 1 etc
					}
				}
				old_id=id;
			}
    }

    // Shutdown the capture graph.
    //CHK( m_pGraphManager->ShutDown());

//Cleanup:
    return 0;
}


void CAppWindow::UpdateMessage( WCHAR *new_message )
{
	int i;

    if( !new_message )
    {
		m_wzStatusString[0] = L'\0';    
	}
    
	for (i=0; new_message[i] != L'\0'; i++)
		m_wzStatusString[i] = new_message[i];

	m_wzStatusString[i] = L'\0';
	
}


void RunVideoStream( void )
{
	HRESULT hr;

	//ICaptureGraphBuilder2 *pBuild;	// Capture Graph Builder

    hr = CoInitialize( NULL );
	hr = InitCaptureGraphBuilder( &pGraph, &g_pBuild );
	hr = InitCaptureFilter( pGraph, pCap );
	hr = InitVideoCapFilter( pGraph, DTMFilter );
	hr = g_pBuild->RenderStream(&PIN_CATEGORY_PREVIEW, &MEDIATYPE_Video, pCap, DTMFilter, NULL);
	EmbedVideoInWindow();
	hr = RunCaptureGraph( g_pBuild );

	//WaitForSingleObject( hwnd, INFINITE );
	//Sleep( 10000 );
}


HRESULT InitCaptureGraphBuilder(
  IGraphBuilder **ppGraph,         // Receives the pointer.
  ICaptureGraphBuilder2 **ppBuild  // Receives the pointer.
)
{
    if (!ppGraph || !ppBuild) {
        return E_POINTER;
    }
    IGraphBuilder *pGraph = NULL;
    ICaptureGraphBuilder2 *pBuild = NULL;

    // Create the Capture Graph Builder.
    HRESULT hr = CoCreateInstance(CLSID_CaptureGraphBuilder, NULL, 
        CLSCTX_INPROC_SERVER, IID_ICaptureGraphBuilder2, (void**)&pBuild);
    if (SUCCEEDED(hr)) {
        // Create the Filter Graph Manager.
        hr = CoCreateInstance(CLSID_FilterGraph, 0, CLSCTX_INPROC_SERVER,
            IID_IGraphBuilder, (void**)&pGraph);
        if (SUCCEEDED(hr)) {
            // Initialize the Capture Graph Builder.
            pBuild->SetFiltergraph(pGraph);

			//CHK( pGraph->RegisterNotificationWindow( m_hwnd ));

            // Return both interface pointers to the caller.
            *ppBuild = pBuild;
            *ppGraph = pGraph; // The caller must release both interfaces.
            return S_OK;
        }
        else {
            pBuild->Release();
        }
    }
    return hr; // Failed
}


HRESULT GetFirstCameraDriver( WCHAR *pwzName ) 
{
  HRESULT hr = S_OK;
  HANDLE handle = NULL;
  DEVMGR_DEVICE_INFORMATION di;
  GUID guidCamera = { 0xCB998A05, 0x122C, 0x4166, 0x84, 0x6A,
                      0x93, 0x3E, 0x4D, 0x7E, 0x3C, 0x86 };

  if( pwzName == NULL ) {
    return E_POINTER;
  }

  di.dwSize = sizeof(di);

  handle = FindFirstDevice( DeviceSearchByGuid, &guidCamera, &di );
  if(( handle == NULL ) || ( di.hDevice == NULL )) {
    ERR( HRESULT_FROM_WIN32( GetLastError() ));
  }

  StringCchCopy( pwzName, MAX_PATH, di.szLegacyName );
 
Cleanup:
  FindClose( handle );
  return hr;
}


// This makes the hardware capture source filter.
HRESULT InitCaptureFilter (
							IGraphBuilder *pGraph,         // The filter graph.
							CComPtr<IBaseFilter> &pCap      // Video capture filter.
					      )
{
    HRESULT hr = S_OK;

	CComVariant   varCamName;
    CPropertyBag  PropBag;
	WCHAR	      wzDeviceName[ MAX_PATH + 1 ];
    CComPtr<IPersistPropertyBag>    pPropertyBag;

    // Create and initialize the video capture filter

    CHK( pCap.CoCreateInstance( CLSID_VideoCapture ) );
    CHK( pCap.QueryInterface( &pPropertyBag ) );


    // We are loading the driver CAM1 in the video capture filter. 
	CHK( GetFirstCameraDriver( wzDeviceName ) );
    varCamName = wzDeviceName;
    if( varCamName.vt != VT_BSTR )
    {
        ERR( E_OUTOFMEMORY );
    }

    CHK( PropBag.Write( L"VCapName", &varCamName ) );   
    CHK( pPropertyBag->Load( &PropBag, NULL ) );

    // Everything succeeded, the video capture filter is added to the filtergraph
    CHK( pGraph->AddFilter( pCap, L"Video Capture Filter Source" ) );


Cleanup:

	return hr;
}


HRESULT InitVideoCapFilter (
						 IGraphBuilder *pGraph,				// The filter graph.
						 CComPtr<IBaseFilter> &DTMFilter	// VideoCap filter.
					   )
{
    HRESULT hr = S_OK;

    CHK( DTMFilter.CoCreateInstance( CLSID_VideoCapFilter ) );
    CHK( pGraph->AddFilter( DTMFilter, L"VideoCapFilter" ) );

	m_VideoCapFilter = DTMFilter; 

Cleanup:

	return hr;
}


HRESULT RunCaptureGraph( ICaptureGraphBuilder2 *pBuild )
{
    HRESULT hr = S_OK;

    CComPtr<IGraphBuilder> pGraphBuilder;
    CComPtr<IMediaControl> pMediaControl;

    // Retrieve the filtergraph off the capture graph builder
    CHK( pBuild->GetFiltergraph( &pGraphBuilder ));

    // Get the media control interface, and run the graph
    CHK( pGraphBuilder->QueryInterface( &pMediaControl ));
    CHK( pMediaControl->Run());

Cleanup:

	return hr;
}


void StudyFilterPins( CComPtr<IBaseFilter> &pFilter )
{
	IEnumPins       *pin_enumerator;
	IPin            *pins;
	ULONG            num_pins;
	IEnumMediaTypes *media_type_enumerator;
	AM_MEDIA_TYPE   *media_types;
	ULONG            num_media_types;

	pFilter->EnumPins( &pin_enumerator );
	pin_enumerator->Next( 5, &pins, &num_pins );

	for ( unsigned int p=0; p<num_pins; p++ )
	{
		pins[p].EnumMediaTypes( &media_type_enumerator );
		media_type_enumerator->Next( 25, &media_types, &num_media_types );
	}

	media_type_enumerator->Release();
	pin_enumerator->Release();
}


void EmbedVideoInWindow(void)
{
    CComPtr<IGraphBuilder> pGraph;

    // Retrieve the filtergraph off the capture graph builder
    g_pBuild->GetFiltergraph( &pGraph );

   // Specify the owner window.
    IVideoWindow    *pVidWin = NULL;
    pGraph->QueryInterface(IID_IVideoWindow, (void **)&pVidWin);
    pVidWin->put_Owner((OAHWND)g_hwnd);
    pVidWin->put_WindowStyle( WS_CHILD | WS_CLIPSIBLINGS);

	RECT grc;
	GetClientRect(g_hwnd, &grc);
	//pVidWin->SetWindowPosition(30, 5, 176, 144);
	pVidWin->SetWindowPosition(0, 0, 20, 20);

    // Set the owner window to receive event notices.
    pGraph->QueryInterface(IID_IMediaEventEx, (void **)&pEvent);
    pEvent->SetNotifyWindow((OAHWND)g_hwnd, WM_GRAPHNOTIFY, 0);
}


void CleanUp(void)
{

    CComPtr<IGraphBuilder> pGraph;
	IVideoWindow    *pVidWin = NULL;

    // Retrieve the filtergraph off the capture graph builder
    g_pBuild->GetFiltergraph( &pGraph );

    pGraph->QueryInterface(IID_IVideoWindow, (void **)&pVidWin);
    pVidWin->put_Visible(OAFALSE);
    pVidWin->put_Owner(NULL);

    // Stop the graph.
    pMediaControl->Stop();

    long evCode;
    pEvent->WaitForCompletion(INFINITE, &evCode);

    pMediaControl->Release();
    pEvent->Release();
    //pGraph->Release();

    //pGraph = NULL;
    //pVidWin = NULL;
    //pMediaControl = NULL;
    //pEvent = NULL;

    PostQuitMessage(0);
}

void HandleEvent() 
{
    long evCode, param1, param2;
    HRESULT hr;
    while (hr = pEvent->GetEvent(&evCode, &param1, &param2, 0), SUCCEEDED(hr))
    { 
        hr = pEvent->FreeEventParams(evCode, param1, param2);
        if ((EC_COMPLETE == evCode) || (EC_USERABORT == evCode))
        { 
            CleanUp();
            break;
        } 
    } 
}
