#include <algorithm>
#include <streams.h>
#include "Bitmap.h"
using namespace std;

BitmapLite::BitmapLite( void )
	: buffer( NULL )
{
}

BitmapLite::BitmapLite( int new_width, int new_height)
{
	buffer=new Pixel[new_width*new_height*3];
}

void BitmapLite::allocate_buffer( int new_width, int new_height)
{
	if (buffer!=NULL)
		delete[] buffer;
	
	buffer=new Pixel[new_width*new_height*3];
	
}


BitmapLite::~BitmapLite( void )
{
	delete[] buffer;
}

int BitmapLite::get_width ( void )
{
	return width;
}


int BitmapLite::get_height ( void )
{
	return height;
}

void BitmapLite::set_width ( int w)
{
	width=w;
}


void BitmapLite::set_height ( int h)
{
	height=h;
}
void BitmapLite::set_image_buffer ( Pixel *new_buffer )
{
	buffer = new_buffer;
}

bool BitmapLite::write  ( char *filename)
{
	BITMAPFILEHEADER bitmapFileHeader;
	BITMAPINFOHEADER bitmapInfoHeader;

	int nofColorChannels=3;

	memset (&bitmapFileHeader,0, sizeof(BITMAPFILEHEADER));
	memset (&bitmapInfoHeader,0, sizeof(BITMAPINFOHEADER));

	bitmapFileHeader.bfType=0x4D42; //BM
	bitmapFileHeader.bfSize=sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+nofColorChannels * width * height;
	bitmapFileHeader.bfOffBits=sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);

	bitmapInfoHeader.biWidth = width;
	bitmapInfoHeader.biHeight = height;
	bitmapInfoHeader.biSize = sizeof (BITMAPINFOHEADER);
	bitmapInfoHeader.biCompression = BI_RGB;
	bitmapInfoHeader.biPlanes = 1;
	bitmapInfoHeader.biBitCount = nofColorChannels*8;
	bitmapInfoHeader.biSizeImage = nofColorChannels * width * height;
	bitmapInfoHeader.biClrUsed =0;
	/*if (nofColorChannels==1)
		bitmapInfoHeader.biClrUsed =256;
	else
		bitmapInfoHeader.biClrUsed =0;*/

	bitmapInfoHeader.biClrImportant =0;


	FILE *fp=fopen(filename, "wb");
	if (fp==0) //{printf("Unable to open %s", filename); exit(-1);}
		return FALSE;
	fwrite(&bitmapFileHeader,1,sizeof(BITMAPFILEHEADER),fp);
	fwrite(&bitmapInfoHeader,1,sizeof(BITMAPINFOHEADER),fp);
	unsigned char *rev_buf=(unsigned char *)calloc(width*height*nofColorChannels, sizeof(char));

	for (int y=0; y<height; y++)
		for (int x=0; x<width; x++)
		{
			rev_buf[3*x+y*width*3]=buffer[x+(height-y-1)*width].color.get_b();
			rev_buf[3*x+1+y*width*3]=buffer[x+(height-y-1)*width].color.get_g();
			rev_buf[3*x+2+y*width*3]=buffer[x+(height-y-1)*width].color.get_r();

			//rev_buf[3*x+y*width*3]=buffer[x+(y)*width].color.get_b();
			//rev_buf[3*x+1+y*width*3]=buffer[x+(y)*width].color.get_g();
			//rev_buf[3*x+2+y*width*3]=buffer[x+(y)*width].color.get_r();

		}

	fwrite(rev_buf,1,width*(long)height*nofColorChannels,fp);
	free (rev_buf);
	fclose(fp);
	return TRUE;
}


bool BitmapLite::read  ( char *filename )
{
	/*get the Bitmap file image into an array*/
	BITMAPFILEHEADER bitmapFileHeader;
	BITMAPINFOHEADER bitmapInfoHeader;

	
	FILE *fp=fopen(filename, "rb");
	
	if (fp==0) {
//		printf("WARNING: Unable to open %s\n",fileName); 
		return FALSE;
	}

	fread(&bitmapFileHeader, 1, sizeof(BITMAPFILEHEADER), fp);
	fread(&bitmapInfoHeader, 1, sizeof(BITMAPINFOHEADER), fp);
	//unsigned char *bitmapBits=(unsigned char *)calloc((long)bitmapInfoHeader.biSizeImage,1);
	unsigned char *rev_buf=(unsigned char *)calloc(width*height*3, sizeof(char));
	//fread(&tmp,1, 1, fp);
	//fread(&tmp,1, 1, fp);
	int a= sizeof(BITMAPINFOHEADER);
	int b=sizeof(BITMAPFILEHEADER);

	fread(rev_buf, bitmapInfoHeader.biWidth*(long)bitmapInfoHeader.biHeight*3, sizeof(unsigned char), fp);
	fclose(fp);

	height=bitmapInfoHeader.biHeight;
	width=bitmapInfoHeader.biWidth;

	//bytes_per_pix=bitmap->bmPlanes;
	
	
	for (int y=0; y<height; y++)
		for (int x=0; x<width; x++)
		{
			buffer[x+(height-y-1)*width].color.set_b(rev_buf[3*x+y*width*3]);
			buffer[x+(height-y-1)*width].color.set_g(rev_buf[3*x+1+y*width*3]);
			buffer[x+(height-y-1)*width].color.set_r(rev_buf[3*x+2+y*width*3]);
		}
	free (rev_buf);

	return TRUE;
}

