#ifndef __BITMAPLITEDEFINED__
#define __BITMAPLITEDEFINED__

class BitmapLite
{
public:

	typedef union _Pixel {
		struct {
			private:
				BYTE b;
				BYTE g;
				BYTE r;
			public:
				BYTE get_r ( void ) const { return r; };
				BYTE get_g ( void ) const { return g; };
				BYTE get_b ( void ) const { return b; };
				void set_r ( BYTE new_r ) { r = new_r; };
				void set_g ( BYTE new_g ) { g = new_g; };
				void set_b ( BYTE new_b ) { b = new_b; };
		} color;

		struct {
			private:
				BYTE value;
			public:
				BYTE get_value ( void ) const     { return value; };
				void set_value ( BYTE new_value ) { value = new_value; };
		} gray;

	} Pixel;

	typedef struct _Iterator {

		private:

			friend class CBitmap;
			Pixel *sample_pixel;
			int row_step;
			int col_step;
			_Iterator( Pixel *_sample_pixel, int _row_step, int _col_step ): 
				sample_pixel(_sample_pixel), row_step(_row_step), col_step(_col_step) {};

		public:

			_Iterator( void ) {};

			Pixel &operator* ( void )       { return *sample_pixel; };
			Pixel *operator->( void ) const { return  sample_pixel; };

			bool operator== ( const _Iterator &other ) { return (sample_pixel == other.sample_pixel); }
			bool operator!= ( const _Iterator &other ) { return (sample_pixel != other.sample_pixel); }

			_Iterator operator++ ( int ) { sample_pixel++;  return *this; }
			_Iterator operator-- ( int ) { sample_pixel--;  return *this; }
			_Iterator &operator+= ( int offset ) { sample_pixel += offset;  return *this; }
			_Iterator &operator-= ( int offset ) { sample_pixel -= offset;  return *this; }

			void prev_row ( void )    { sample_pixel -= col_step; };
			void next_row ( void )    { sample_pixel += col_step; };
			void prev_col ( void )    { sample_pixel -= row_step; };
			void next_col ( void )    { sample_pixel += row_step; };

	} Iterator;


	BitmapLite( void );
	BitmapLite( int new_width, int new_height );
	~BitmapLite( void );

	bool BitmapLite::write  ( char *filename);
	bool BitmapLite::read  ( char *filename);
	void BitmapLite::set_image_buffer ( Pixel *new_buffer );
	void BitmapLite::allocate_buffer( int new_width, int new_height);

	int  get_width  ( void );
	int  get_height ( void );
	void  set_width  ( int w );
	void  set_height ( int h );
	void set_buffer ( Pixel *new_buffer );
	Pixel *get_pixel ( int col_index, int row_index );
	
	Iterator get_iterator ( int linear_index );
	Iterator get_iterator ( int col_index, int row_index );
	Iterator begin     ( void );
	Iterator begin_row ( int row_index );
	Iterator begin_col ( int col_index );
	Iterator end       ( void );
	Iterator end_row   ( int row_index );
	Iterator end_col   ( int col_index );

	Pixel *buffer;
	BYTE *yuvP;


private:

	int width, height;
};

#endif