#include "CImg.h"
using namespace cimg_library;

#include <iostream>
using namespace std;

BYTE gGaussianFilter5x5[5][5]  ={1,2,2,2,1,
                                 2,4,4,4,2,
                                 2,4,5,4,2,
                                 2,4,4,4,2,
								 1,2,2,2,1};




template<typename t>
void applyEBMA(CImg<t> &inAnchorIm, CImg<t> &inTargetIm,
			   CImg<int> &inMotion,
			   const int inBlockSize,const int inRadius)
{
	//Gaussian filter the images
	CImg<BYTE> fGaussian5x5; fGaussian5x5.assign(&gGaussianFilter5x5[0],5,5,1,1);
	CImg<long> tAnchorIm = CImg<long>(inAnchorIm).convolve(fGaussian5x5)>>6;
	CImg<long> tTargetIm = CImg<long>(inTargetIm).convolve(fGaussian5x5)>>6;

	//Calculate the image mean values
	long tAncBlkMean = (long)tAnchorIm.sum() / ((long)tAnchorIm.width*(long)tAnchorIm.height); 
	long tTarBlkMean = (long)tTargetIm.sum() / ((long)tTargetIm.width*(long)tTargetIm.height);

	//CImgDisplay cDisplay1(tTargetIm.width,tTargetIm.height,"Smoothed Target");
	//cDisplay1.display(inTargetIm);
	//cDisplay1.wait(10);
	//cin.get();

	
	unsigned int tWidth  = inAnchorIm.width;
	unsigned int tHeight = inAnchorIm.height;

	for(unsigned int y=0; y<tHeight; y+=inBlockSize){
	for(unsigned int x=0; x<tWidth;  x+=inBlockSize){
		
		//crop the blocks
		//CImg<int> tAnchorBlk = CImg<int>(CImg<BYTE>(*inAnchorIm).crop(x,y,x+inBlockSize-1,y+inBlockSize-1));
		//CImg<BYTE> tAnchorBlk = CImg<BYTE>(CImg<BYTE>(*inAnchorIm).crop(x,y,x+inBlockSize-1,y+inBlockSize-1));

		//shift to previous estimate
		int px = x + inMotion(x/inBlockSize,y/inBlockSize,0,0);
		int py = y + inMotion(x/inBlockSize,y/inBlockSize,0,1);


		long tMinMAD = 255*inBlockSize*inBlockSize*3;
		int  dx,dy; dx = dy =0;
		for(int ry = -inRadius; ry<=inRadius; ry++){
		for(int rx = -inRadius; rx<=inRadius; rx++){
		

			if( (px+rx)>=0 && (px+rx)<=(tWidth-inBlockSize) && 
				(py+ry)>=0 && (py+ry)<=(tHeight-inBlockSize)){
				
				//CImg<BYTE> tTargetBlk = CImg<BYTE>(*inTargetIm).crop(px+rx,py+ry,px+rx+inBlockSize-1,py+ry+inBlockSize-1);
		/*		
				//long tMAD = CImg<int>(tTargetBlk - tAnchorBlk).abs().sum();
				long tAncBlkMean,tTarBlkMean;
				tAncBlkMean = tTarBlkMean = 0;

				//find mean values of blocks
				for(int k=0;k<inAnchorIm.dim;k++){
				for(int j=0;j<inBlockSize;j++){
					long *sptrA = &(tAnchorIm.data[tAnchorIm.offset(x,y+j,0,k)]);
					long *sptrT = &(tTargetIm.data[tTargetIm.offset(px+rx,py+ry+j,0,k)]);
				for(int i=0;i<inBlockSize;i++){
					//Slow but readable
					//tAncBlkMean += tAnchorIm(x+i,y+j,0,k);
					//tTarBlkMean += tTargetIm(px+rx+i,py+ry+j,0,k);
					
					//A bit faster harder to read
					tAncBlkMean += (*sptrA++);
					tTarBlkMean += (*sptrT++);
					//tMAD += abs((int)(*sptrA++) - (int)(*sptrT++));
				}
				}
				}
				tAncBlkMean /= inBlockSize*inBlockSize;
				tTarBlkMean /= inBlockSize*inBlockSize;
			*/
				long tMAD = 0;
				for(int k=0;k<tAnchorIm.dim;k++){
				for(int j=0;j<inBlockSize;j++){
					long *sptrA = &(tAnchorIm.data[tAnchorIm.offset(x,y+j,0,k)]);
					long *sptrT = &(tTargetIm.data[tTargetIm.offset(px+rx,py+ry+j,0,k)]);
				for(int i=0;i<inBlockSize;i++){
					//tMAD += abs((int)tAnchorIm(x+i,y+j,0,k) - (int)tTargetIm(px+rx+i,py+ry+j,0,k));
					//Subtract mean values to compansate lighting noise
					//tMAD += abs((int)tAnchorIm(x+i,y+j,0,k) - (int)tTargetIm(px+rx+i,py+ry+j,0,k)-tAncBlkMean+tTarBlkMean);
					tMAD += abs((int)(*sptrA++) - (int)(*sptrT++)-tAncBlkMean+tTarBlkMean);
				}
				}
				}
				

				if (tMAD<tMinMAD){
					tMinMAD = tMAD;
					dx = rx;
					dy = ry;					
				}
				
			}
		}
		}

		inMotion(x/inBlockSize,y/inBlockSize,0,0) += dx;
		inMotion(x/inBlockSize,y/inBlockSize,0,1) += dy;

	}
	}
}

template<typename t>
CImg<int> applyHBMA(CImg<t> &inAnchorIm, CImg<t> &inTargetIm,
			   const int inBlockSize,const int inRadius,const int inLevels)
{
	int tWidth, tHeight, tDepth, tDim, tRadius;

	long tResizeFactor = 1<<inLevels;
	CImg<int> outMotion = CImg<int>(inAnchorIm.width/(tResizeFactor*inBlockSize), inAnchorIm.height/(tResizeFactor*inBlockSize),1,2,0);

	for(int n = inLevels; n>2; n--){
		
		tResizeFactor = 1<<n;		

		//width and height of the image
		tWidth	= inAnchorIm.width  / tResizeFactor;
		tHeight	= inAnchorIm.height / tResizeFactor;		
		tDepth  = inAnchorIm.depth;
		tDim	= inAnchorIm.dim ;

		//copy and resize the image
		CImg<t> tAnchorIm = CImg<t>(inAnchorIm).resize(tWidth,tHeight,tDepth,tDim,6);
		CImg<t> tTargetIm = CImg<t>(inTargetIm).resize(tWidth,tHeight,tDepth,tDim,6);
		/*
		//check the block size and zeros padding need		
		if(tAnchorIm.width % inBlockSize > 0)
			tWidth = (tAnchorIm.width / inBlockSize + 1 ) * inBlockSize;
		if(tAnchorIm.height % inBlockSize > 0)
			tHeight = (tAnchorIm.height / inBlockSize +1 ) * inBlockSize;
		if(tAnchorIm.width != tWidth && tAnchorIm.height != tHeight){
			tAnchorIm.resize(tWidth,tHeight,tAnchorIm.depth,tAnchorIm.dim,6);
			tTargetIm.resize(tWidth,tHeight,tTargetIm.depth,tTargetIm.dim,6);
		}
		*/

		//Search radius
		tRadius = inRadius / (1<<n);

		//Correct the motion vectors
		applyEBMA(tAnchorIm,tTargetIm,outMotion,inBlockSize,tRadius);

		//Upsample the blocks
		outMotion.resize(outMotion.width*2,outMotion.height*2,outMotion.depth,outMotion.dim,1);
		//tOutMotion.resize(tOutMotion.width*2,tOutMotion.height*2,tOutMotion.depth,tOutMotion.dim,1);

	}
	return outMotion;
}
template<typename t>
CImg<int> blockConfidence(CImg<t> &inTestIm, const int inBlockSize, const int inThresh = 3 ){
	
	CImg<int> outConfidence = CImg<int>(inTestIm.width/inBlockSize,inTestIm.height/inBlockSize,1,2,0);

	CImg<BYTE> fGaussian5x5; fGaussian5x5.assign(&gGaussianFilter5x5[0],5,5,1,1);
	CImg<long> tTestIm = CImg<long>(inTestIm).convolve(fGaussian5x5)>>6;
	//gradient images
	CImg<int> tGradientImX = CImg<int>(tTestIm).get_gradientX();
	CImg<int> tGradientImY = CImg<int>(tTestIm).get_gradientY();
/*
	tGradientImX.abs();
	tGradientImY.abs();
*/

	
	//calculate the gradient points
	tGradientImX.abs().threshold(inThresh);
	tGradientImY.abs().threshold(inThresh);
	

	for(unsigned int y=inBlockSize; y<inTestIm.height-inBlockSize; y+=inBlockSize){
	for(unsigned int x=inBlockSize; x<inTestIm.width-inBlockSize; x+=inBlockSize){

		//gradient through directions
		int dx,dy;
		dx = dy = 0;
		for(int j=0;j<inBlockSize-1;j++){
		for(int i=0;i<inBlockSize-1;i++){
			dx += tGradientImX(x+i,y+j,0,0);
			dy += tGradientImY(x+i,y+j,0,0);
		}
		}
		outConfidence(x/inBlockSize,y/inBlockSize,0,0) = dx;
		outConfidence(x/inBlockSize,y/inBlockSize,0,1) = dy;
	}
	}
	
/*
	for(int v=0;v<outConfidence.dim;v++){
	for(int y=0;y<outConfidence.height;y++){
	for(int x=0;x<outConfidence.width;x++){
		outConfidence(x,y,0,v) = rand();
		cout.width(5);
		cout<<"("<<outConfidence(x,y,0,v)<<","<<outConfidence(x,y,0,v)<<") ";
	}
	cout<<endl;
	}
	cout<<endl<<endl;
	}
	*/
	
	return outConfidence;
}